# -*- coding: utf-8 -*-

from .base import BaseRESTAPI
from .group_open_http_svc import GroupOpenHttpSVC  # 群组管理
from .im_open_login_svc import IMOpenLoginSVC  # 帐号管理
from .open_im import OpenIM  # 单聊消息
from .profile import Profile  # 资料管理
from .sns import SNS  # 关系链管理
from .open_config_svr import OpenConfigSVR  # 运营管理
from .open_msg_svc import OpenMSGSVC  # 消息记录



# -*- coding: utf-8 -*-


from . import BaseRESTAPI


class SNS(BaseRESTAPI):
    """ 关系链管理

    """

    ADD_TYPE_BOTH = 'Add_Type_Both'
    ADD_TYPE_SINGLE = 'Add_Type_Single'

    FORCE_ADD_FLAGS_YES = 1
    FORCE_ADD_FLAGS_NO = 0

    DELETE_TYPE_SINGLE = 'Delete_Type_Single'
    DELETE_TYPE_BOTH = 'Delete_Type_Both'

    CHECK_RESULT_TYPE_SINGLE = 'CheckResult_Type_Single'
    CHECK_RESULT_TYPE_BOTH = 'CheckResult_Type_Both'
    BLACK_CHECK_RESULT_TYPE_SINGLE = 'BlackCheckResult_Type_Single'
    BLACK_CHECK_RESULT_TYPE_BOTH = 'BlackCheckResult_Type_Both'

    def friend_add(self, from_account, add_friend_item, add_type=ADD_TYPE_BOTH, force_add_flags=FORCE_ADD_FLAGS_YES):
        """添加好友

        https://cloud.tencent.com/document/product/269/1643
        """
        data = {
            'From_Account': from_account,
            'AddFriendItem': add_friend_item,
            'AddType': add_type,
            'ForceAddFlags': force_add_flags
        }

        return self._post(
            'sns/friend_add',
            data=data
        )

    def friend_import(self, from_account, add_friend_item):
        """导入好友

        https://cloud.tencent.com/document/product/269/8301
        """
        data = {
            'From_Account': from_account,
            'AddFriendItem': add_friend_item,
        }

        return self._post(
            'sns/friend_import',
            data=data
        )

    def friend_update(self, from_account, update_item):
        """更新好友

        https://cloud.tencent.com/document/product/269/12525
        """
        data = {
            'From_Account': from_account,
            'UpdateItem': update_item,
        }

        return self._post(
            'sns/friend_update',
            data=data
        )

    def friend_delete(self, from_account, to_account, delete_type=DELETE_TYPE_BOTH):
        """删除好友

        https://cloud.tencent.com/document/product/269/1644
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
            'DeleteType': delete_type,
        }

        return self._post(
            'sns/friend_delete',
            data=data
        )

    def friend_delete_all(self, from_account, delete_type=DELETE_TYPE_BOTH):
        """删除所有好友

        https://cloud.tencent.com/document/product/269/1645
        """
        data = {
            'From_Account': from_account,
            'DeleteType': delete_type,
        }

        return self._post(
            'sns/friend_delete_all',
            data=data
        )

    def friend_check(self, from_account, to_account, check_type=CHECK_RESULT_TYPE_BOTH):
        """校验好友

        https://cloud.tencent.com/document/product/269/1645
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
            'CheckType': check_type
        }

        return self._post(
            'sns/friend_check',
            data=data
        )

    def friend_get(self, from_account, start_index=0, standard_sequence=None, custom_sequence=None):
        """拉取好友

        https://cloud.tencent.com/document/product/269/1647
        """
        data = {
            'From_Account': from_account,
            'StartIndex': start_index,
        }
        if standard_sequence:
            data['StandardSequence']: standard_sequence
        if custom_sequence:
            data['StandardSequence']: custom_sequence

        return self._post(
            'sns/friend_get',
            data=data
        )

    def friend_get_list(self, from_account, to_account, tag_list=None):
        """拉取指定好友

        https://cloud.tencent.com/document/product/269/8609
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
            'TagList': tag_list or [
                "Tag_Profile_IM_Image",
                "Tag_Profile_IM_Nick",
                "Tag_SNS_IM_Remark",
                "Tag_SNS_IM_Group"
            ],
        }

        return self._post(
            'sns/friend_get_list',
            data=data
        )

    def black_list_add(self, from_account, to_account):
        """添加黑名单

        https://cloud.tencent.com/document/product/269/3718
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
        }

        return self._post(
            'sns/black_list_add',
            data=data
        )

    def black_list_delete(self, from_account, to_account):
        """删除黑名单

        https://cloud.tencent.com/document/product/269/3719
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
        }

        return self._post(
            'sns/black_list_delete',
            data=data
        )

    def black_list_get(self, from_account, start_index=0, max_limited=30, last_sequence=0):
        """拉取黑名单

        https://cloud.tencent.com/document/product/269/3719
        """
        data = {
            'From_Account': from_account,
            'StartIndex': start_index,
            'MaxLimited': max_limited,
            'LastSequence': last_sequence
        }

        return self._post(
            'sns/black_list_get',
            data=data
        )

    def black_list_check(self, from_account, to_account, check_type=BLACK_CHECK_RESULT_TYPE_BOTH):
        """校验黑名单

        https://cloud.tencent.com/document/product/269/3725
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
            'CheckType': check_type,
        }

        return self._post(
            'sns/black_list_check',
            data=data
        )

    def group_add(self, from_account, group_name, to_account=None):
        """添加分组

        https://cloud.tencent.com/document/product/269/10107
        """
        data = {
            'From_Account': from_account,
            'GroupName': group_name,
        }
        if to_account:
            data['To_Account'] = to_account

        return self._post(
            'sns/group_add',
            data=data
        )

    def group_delete(self, from_account, group_name):
        """删除分组

        https://cloud.tencent.com/document/product/269/10108
        """
        data = {
            'From_Account': from_account,
            'GroupName': group_name,
        }

        return self._post(
            'sns/group_delete',
            data=data
        )



# -*- coding: utf-8 -*-
from enum import IntEnum, unique


@unique
class RESSTAPIErrorCode(IntEnum):
    """
    REST API 公共错误码，请参考 https://cloud.tencent.com/document/product/269/1519
    IM SDK 的错误码 + 服务端的错误码+ IM SDK V3 版本的错误码https://cloud.tencent.com/document/product/269/1671
    """
    # 请求包非法
    BODY_ILLEGAL_20001 = 20001

    # UserSig 或 A2 失效
    USER_SIG_OR_A2_INVALID_20002 = 20002

    # 消息发送方或接收方 UserID 无效或不存在，请检查 UserID 是否已导入即时通信 IM
    TO_ACCOUNT_OR_FROM_ACCOUNT_USER_ID_INVALID_20003 = 20003

    # 网络异常，请重试
    NETWORK_ERROR_20004 = 20004

    # 服务器内部错误，请重试
    INTERNAL_ERROR_20005 = 20005

    # 触发发送单聊消息之前回调，App 后台返回禁止下发该消息
    MESSAGE_FORBIDDEN_20006 = 20006

    # 清理关系链数据时服务器内部出错，请稍后重试
    INTERNAL_ERROR_WHEN_DELETE_30006 = 30006

    # 清理关系链数据时服务器内部超时，请稍后重试
    INTERNAL_ERROR_WHEN_DELETE_30007 = 30007

    # 并发写关系链数据触发写冲突，建议使用批量方式
    CONFLICT_WHEN_DELETE_30008 = 30008

    # 40001	请求参数错误，请根据错误描述检查请求参数
    BODY_INVALID_40001 = 40001

    # 请求参数错误，没有指定 To_Account
    BODY_TO_ACCOUNT_INVALID_40002 = 40002

    # 请求的用户帐号不存在
    USER_NOT_EXIST_40003 = 40003

    # 请求需要 App 管理员权限
    PERMISSIONS_LIMITED_40004 = 40004

    # 资料字段中包含敏感词
    CONTAINED_SENSITIVE_WORD_40005 = 40005

    # 设置资料时服务器内部错误，请稍后重试
    INTERNAL_ERROR_40006 = 40006

    # 资料字段的 Value 长度超过500字节
    VALUE_TOO_LONG_40601 = 40601
    # HTTP 解析错误 ，请检查 HTTP 请求 URL 格式
    HTTP_PARSER_ERROR_60002 = 60002

    # HTTP 请求 JSON 解析错误，请检查 JSON 格式
    BODY_JSON_PARSER_ERROR_60003 = 60003

    # 请求 URL 或 JSON 包体中帐号或签名错误
    REQUEST_AUTHENTICATION_ERROR_60004 = 60004

    # 请求 URL 或 JSON 包体中帐号或签名错误
    REQUEST_AUTHENTICATION_ERROR_60005 = 60005

    # SDKAppID 失效，请核对 SDKAppID 有效性
    SDK_APP_ID_INVALID_60006 = 60006

    # REST 接口调用频率超过限制，请降低请求频率
    REST_API_INVOKE_LIMITED_60007 = 60007

    # 服务请求超时或 HTTP 请求格式错误，请检查并重试
    REQUEST_TIMEOUT_OR_METHOD_ERROR_60008 = 60008

    # 请求资源错误，请检查请求 URL
    RESOURCE_ERROR_60009 = 60009

    # 请求需要 App 管理员权限
    PERMISSIONS_LIMITED_60010 = 60010

    # 请求频率超限，请降低请求频率
    SDK_APP_ID_INVOKE_LIMITED_60011 = 60011

    # REST 接口需要带 SDKAppID，请检查请求 URL 中的 SDKAppID
    SDK_APP_ID_REQUIRE_60012 = 60012

    # HTTP 响应包 JSON 解析错误
    RESPONSE_JSON_DECODE_ERROR_60013 = 60013

    # 置换帐号超时
    ACCOUNT_SWAP_TIMEOUT_60014 = 60014

    # 请求包体帐号类型错误，请确认帐号为字符串格式
    BODY_ACCOUNT_TYPE_ERROR_60015 = 60015

    # 请求删除的  不存在，请确认 UserID 的合法性
    USER_ID_NOT_EXIST_70107 = 70107

    # 服务端内部超时，请稍后重试
    INTERNAL_TIMEOUT_70169 = 70169

    # 服务端内部超时，请稍后重试
    INTERNAL_ERROR_70202 = 70202

    # 帐号数超限。如需创建多于100个帐号，请将应用升级为专业版，具体操作指引请参见 购买指引
    NUMBER_OF_ACCOUNTS_LIMITED_70398 = 70398

    # 参数非法，请检查必填字段是否填充，或者字段的填充是否满足协议要求
    PARAM_ILLEGAL_70402 = 70402

    # 请求失败，需要 App 管理员权限
    PERMISSIONS_LIMITED_70403 = 70403

    # 服务器内部错误，请稍后重试
    INTERNAL_ERROR_70500 = 70500

    # 删除帐号失败。仅支持删除体验版帐号，您当前应用为专业版，暂不支持帐号删除
    ACCOUNT_DELETE_FAIL_71000 = 71000

    # 消息文本安全打击
    MESSAGE_TEXT_SECURITY_STRIKE_80001 = 80001

    # JSON 格式解析失败，请检查请求包是否符合 JSON 规范
    BODY_JSON_PARSER_ERROR_90001 = 90001

    # JSON 格式请求包中 MsgBody 不符合消息格式描述，或者 MsgBody 不是 Array 类型，请参考 TIMMsgElement 对象 的定义
    BODY_MSG_BODY_INVALID_90002 = 90002

    # JSON 格式请求包体中缺少 To_Account 字段或者 To_Account 字段不是 String 类型
    BODY_TO_ACCOUNT_INVALID_90003 = 90003

    # JSON 格式请求包体中缺少 MsgRandom 字段或者 MsgRandom 字段不是 Integer 类型
    BODY_MSG_RANDOM_INVALID_90005 = 90005

    # JSON 格式请求包体中 MsgTimeStamp 字段不是 Integer 类型
    BODY_MSG_TIMESTAMP_INVALID_90006 = 90006

    # JSON 格式请求包体中 MsgBody 类型不是 Array 类型，请将其修改为 Array 类型
    BODY_MSG_BODY_INVALID_90007 = 90007

    # JSON 格式请求包体中缺少 From_Account 字段，或者From_Account 不存在。
    BODY_FROM_ACCOUNT_INVALID_90008 = 90008

    # 请求需要 App 管理员权限
    PERMISSIONS_LIMITED_90009 = 90009

    # JSON 格式请求包不符合消息格式描述，请参考 TIMMsgElement 对象 的定义
    BODY_TIM_MSG_ELEMENT_INVALID_90010 = 90010

    # 批量发消息目标帐号超过500，请减少 To_Account 中目标帐号数量
    NUMBER_OF_TO_ACCOUNT_LIMITED_90011 = 90011

    # To_Account 没有注册或不存在，请确认 To_Account 是否导入即时通信 IM 或者是否拼写错误
    BODY_TO_ACCOUNT_INVALID_90012 = 90012

    # 消息离线存储时间错误（最多不能超过7天）
    BODY_MSG_LIFETIME_INVALID_90026 = 90026

    # JSON 格式请求包体中缺少 SyncFromOldSystem 字段或者 SyncFromOldSystem 字段不是 Integer 类型
    BODY_SYNC_FROM_OLD_SYSTEM_INVALID_90030 = 90030

    # JSON 格式请求包体中 SyncOtherMachine 字段不是 Integer 类型
    BODY_SYNC_OTHER_MACHINE_INVALID_90031 = 90031

    # JSON 格式请求包体中 MsgLifeTime 字段不是 Integer 类型
    BODY_MSG_LIFETIME_INVALID_90044 = 90044

    # 请求的用户帐号不存在
    ACCOUNT_NOT_EXIST_90048 = 90048

    # 撤回请求中的 MsgKey 不合法
    BODY_MSG_KEY_INVALID_90054 = 90054

    # 服务内部错误，请重试；
    # 如果所有请求都返回该错误码，且 App 配置了第三方回调，请检查 App 服务器是否正常向即时通信 IM 后台服务器返回回调结果
    INTERNAL_ERROR_90992 = 90992

    # 服务内部错误，请重试
    INTERNAL_ERROR_90994 = 90994

    # 服务内部错误，请重试
    INTERNAL_ERROR_90995 = 90995

    # 服务内部错误，请重试
    INTERNAL_ERROR_91000 = 91000

    # JSON 数据包超长，消息包体请不要超过 8k
    BODY_TOO_LONG_93000 = 93000



# -*- coding: utf-8 -*-
import time

from . import BaseRESTAPI


class OpenMSGSVC(BaseRESTAPI):
    """消息记录

    """
    CHAT_TYPE_C2C = 'C2C'
    CHAT_TYPE_GROUP = 'Group'

    def get_history(self, chat_type, msg_time=None):
        """下载消息记录

        https://cloud.tencent.com/document/product/269/1650
        """
        data = {
            'ChatType': chat_type,
            'MsgTime': msg_time or time.strftime('%Y%m%d%H', time.localtime(time.time() - 60 * 60 * 3))
        }

        return self._post(
            'open_msg_svc/get_history',
            data=data
        )



# -*- coding: utf-8 -*-

from . import BaseRESTAPI


class IMOpenLoginSVC(BaseRESTAPI):
    """帐号管理

    """

    def account_import(self, identifier, nick=None, face_url=None):
        """单个帐号导入接口

        https://cloud.tencent.com/document/product/269/1608
        """
        data = {
            'Identifier': identifier
        }
        if nick:
            data['Nick'] = nick
        if face_url:
            data['FaceUrl'] = face_url

        return self._post(
            'im_open_login_svc/account_import',
            data=data
        )

    def im_open_login_svc(self, accounts):
        """批量帐号导入接口

        https://cloud.tencent.com/document/product/269/4919
        """
        return self._post(
            'im_open_login_svc/multiaccount_import',
            data={
                'Accounts': accounts
            }
        )

    def account_delete(self, delete_item):
        """帐号删除接口

        https://cloud.tencent.com/document/product/269/36443
        """
        return self._post(
            'im_open_login_svc/account_delete',
            data={
                'DeleteItem': [{'UserID': user_id} for user_id in delete_item]
            }
        )

    def account_check(self, check_item):
        """帐号检查接口

        https://cloud.tencent.com/document/product/269/38417
        """
        return self._get(
            'im_open_login_svc/account_check',
            data={
                'CheckItem': [{'UserID': user_id} for user_id in check_item]
            }
        )

    def kick(self, identifier):
        """帐号登录态失效接口

        https://cloud.tencent.com/document/product/269/3853
        """
        return self._post(
            'im_open_login_svc/kick',
            data={
                'Identifier': identifier
            }
        )



# -*- coding: utf-8 -*-

# Author: Severus Well
# Contact: thinkweiwei@msn.com

import json
import random
import requests


class HttpClient(object):

    def __init__(self, app_id, identifier, signature, timeout=10, auto_retry=True):
        self._app_id = app_id
        self._identifier = identifier
        self._signature = signature
        self.timeout = timeout
        self.auto_retry = auto_retry

        self._http = requests.Session()

    def get(self, path, **kwargs):
        return self._request(
            method='get',
            url=self.get_url(path),
            **kwargs
        )

    def post(self, path, **kwargs):
        return self._request(
            method='post',
            url=self.get_url(path),
            **kwargs
        )

    def get_url(self, path, random_id=random.randint(0, 4294967295)):
        protocol = 'https'
        host = 'console.tim.qq.com'
        ver = 'v4'
        url = '{protocol}://{host}/{ver}/{path}?sdkappid={app_id}&identifier={identifier}&usersig={signature}' \
              '&random={random_id}&contenttype=json'.format(
            protocol=protocol, host=host, ver=ver, path=path, app_id=self._app_id, identifier=self._identifier,
            signature=self._signature, random_id=random_id)
        return url

    def _request(self, method, url, **kwargs):
        if 'params' not in kwargs:
            kwargs['params'] = {}

        if isinstance(kwargs.get('data', ''), dict):
            body = (json.dumps(kwargs['data'], ensure_ascii=False)).encode('utf-8')
            kwargs['data'] = body

        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        result_processor = kwargs.pop('result_processor', None)
        res = self._http.request(method=method, url=url, **kwargs)

        try:
            res.raise_for_status()
        except Exception as e:
            raise Exception('TIM REST API request error!')

        return self._handle_result(res, method, url, result_processor, **kwargs)

    def _handle_result(self, res, method=None, url=None, result_processor=None, **kwargs):
        if not isinstance(res, dict):  # Dirty hack around asyncio based AsyncWeChatClient
            result = self._decode_result(res)
        else:
            result = res

        if not isinstance(result, dict):
            return result

        return result if not result_processor else result_processor(result)

    @staticmethod
    def _decode_result(res):
        try:
            result = json.loads(res.content.decode('utf-8', 'ignore'), strict=False)
        except (TypeError, ValueError):
            # Return origin response object if we can not decode it as JSON
            print('Can not decode response as JSON!')
            return res
        return result



# -*- coding: utf-8 -*-


from . import BaseRESTAPI


class OpenConfigSVR(BaseRESTAPI):
    """运营管理/全局禁言管理

    """

    def set_no_speaking(self, to_account, c2c_msg_no_speaking_time=None, group_msg_no_speaking_time=None):
        """设置全局禁言

        https://cloud.tencent.com/document/product/269/4230
        """
        data = {
            'Set_Account': to_account
        }
        if c2c_msg_no_speaking_time:
            data['C2CmsgNospeakingTime'] = c2c_msg_no_speaking_time
        if group_msg_no_speaking_time:
            data['GroupmsgNospeakingTime'] = group_msg_no_speaking_time

        return self._post(
            'openconfigsvr/setnospeaking',
            data=data
        )

    def get_no_speaking(self, get_account):
        """查询全局禁言

        https://cloud.tencent.com/document/product/269/4229
        """
        data = {
            'Get_Account': get_account
        }

        return self._post(
            'openconfigsvr/getnospeaking',
            data=data
        )

    def get_app_info(self, request_field=None):
        """拉取运营数据

        https://cloud.tencent.com/document/product/269/4193
        """
        data = {
        }
        if request_field:
            data['RequestField'] = request_field

        return self._post(
            'openconfigsvr/getappinfo',
            data=data
        )



# -*- coding: utf-8 -*-
import random
import time

from . import BaseRESTAPI


class OpenIM(BaseRESTAPI):
    """单聊消息

    """
    SYNC_OTHER_MACHINE_YES = 1
    SYNC_OTHER_MACHINE_NO = 2

    SYNC_FROM_OLD_SYSTEM_REAL_TIME = 1
    SYNC_FROM_OLD_SYSTEM_HISTORY = 2

    def send_text_msg(self, to_account, content, from_account=None, sync_other_machine=SYNC_OTHER_MACHINE_YES,
                      msg_life_time=604800):
        """单发单聊文本消息

        https://cloud.tencent.com/document/product/269/2282
        """
        data = {
            'SyncOtherMachine': sync_other_machine,
            'To_Account': to_account,
            'MsgLifeTime': msg_life_time,
            'MsgRandom': random.randint(0, 4294967295),
            'MsgTimeStamp': int(time.time()),
            "MsgBody": [
                {
                    "MsgType": "TIMTextElem",
                    "MsgContent": {
                        "Text": content
                    }
                }
            ]
        }
        if from_account:
            data['From_Account'] = from_account

        return self._post(
            'openim/sendmsg',
            data=data
        )

    def batch_send_text_msg(self, to_account, content, from_account=None, sync_other_machine=SYNC_OTHER_MACHINE_YES):
        """批量发单聊消息

        https://cloud.tencent.com/document/product/269/1612
        """
        data = {
            'SyncOtherMachine': sync_other_machine,
            'To_Account': to_account,
            'MsgRandom': random.randint(0, 4294967295),
            "MsgBody": [
                {
                    "MsgType": "TIMTextElem",
                    "MsgContent": {
                        "Text": content
                    }
                }
            ]
        }
        if from_account:
            data['From_Account'] = from_account

        return self._post(
            'openim/batchsendmsg',
            data=data
        )

    def import_text_msg(self, to_account, from_account, content, msg_time_stamp=random.randint(0, 4294967295),
                        sync_from_old_system=SYNC_FROM_OLD_SYSTEM_REAL_TIME):
        """导入单聊消息

        https://cloud.tencent.com/document/product/269/2568
        """
        data = {
            'SyncFromOldSystem': sync_from_old_system,
            'From_Account': from_account,
            'To_Account': to_account,
            'MsgRandom': msg_time_stamp,
            'MsgTimeStamp': int(time.time()),
            "MsgBody": [
                {
                    "MsgType": "TIMTextElem",
                    "MsgContent": {
                        "Text": content
                    }
                }
            ]
        }

        return self._post(
            'openim/importmsg',
            data=data
        )

    def admin_msg_withdraw(self, to_account, from_account, msg_key):
        """撤回单聊消息

        https://cloud.tencent.com/document/product/269/38980
        """
        data = {
            'From_Account': from_account,
            'To_Account': to_account,
            'MsgKey': msg_key,
        }

        return self._post(
            'openim/admin_msgwithdraw',
            data=data
        )

    def query_state(self, to_account):
        """获取用户在线状态

        https://cloud.tencent.com/document/product/269/2566
        """
        data = {
            'To_Account': to_account,
        }

        return self._post(
            'openim/querystate',
            data=data
        )



# -*- coding: utf-8 -*-


from . import BaseRESTAPI


class Profile(BaseRESTAPI):
    """资料管理

    """

    def portrait_get(self, to_account, tag_list=None):
        """拉取资料

        https://cloud.tencent.com/document/product/269/1639
        """
        data = {
            'To_Account': to_account,
            'TagList': tag_list or [
                "Tag_Profile_IM_Nick",
                "Tag_Profile_IM_Gender",
                "Tag_Profile_IM_BirthDay",
                "Tag_Profile_IM_AdminForbidType",
                "Tag_Profile_IM_AllowType",
                "Tag_Profile_IM_SelfSignature",
            ]

        }

        return self._post(
            'profile/portrait_get',
            data=data
        )

    def portrait_set(self, from_account, profile_item):
        """设置资料

        https://cloud.tencent.com/document/product/269/1640
        """
        data = {
            'From_Account': from_account,
            'ProfileItem': profile_item
        }

        return self._post(
            'profile/portrait_set',
            data=data
        )



# -*- coding: utf-8 -*-
import os

from TLSSigAPIv2 import TLSSigAPIv2


class TLSSig(object):
    @staticmethod
    def gen_sig(identifier, app_id=None, app_key=None, expire=180 * 86400):
        app_id = app_id or os.environ.get('TENCENT_IM_app_id')
        app_key = app_key or os.environ.get('TENCENT_IM_SDK_APP_SECRET')

        api = TLSSigAPIv2(app_id, app_key)
        return api.gen_sig(identifier, expire)


if __name__ == "__main__":
    sig = TLSSig.gen_sig('xiaojun')
    print(sig)



# -*- coding: utf-8 -*-


class BaseRESTAPI(object):
    """ REST API base class """

    def __init__(self, client=None):
        self._client = client

    def _get(self, path, **kwargs):
        return self._client.get(path, **kwargs)

    def _post(self, path, **kwargs):
        return self._client.post(path, **kwargs)

    def get_client(self):
        return self._client

    def set_client(self, client):
        self._client = client



# -*- coding: utf-8 -*-

import random
from . import BaseRESTAPI


class GroupOpenHttpSVC(BaseRESTAPI):
    """群组管理

    """
    TYPE_PRIVATE = 'Private'
    TYPE_PUBLIC = 'Public'
    TYPE_CHAT_ROOM = 'ChatRoom'
    TYPE_AV_CHAT_ROOM = 'AVChatRoom'
    TYPE_B_CHAT_ROOM = 'BChatRoom'

    JOIN_OPTION_FREE_ACCESS = 'FreeAccess'
    JOIN_OPTION_NEED_PERMISSION = 'NeedPermission'
    JOIN_OPTION_DISABLE_APPLY = 'DisableApply'

    SHUT_UP_ALL_MEMBER_ON = 'On'
    SHUT_UP_ALL_MEMBER_OFF = 'Off'

    SILENCE_YES = 1
    SILENCE_NO = 0

    ROLE_ADMIN = 'Admin'
    ROLE_MEMBER = 'Member'

    MSG_FLAG_ACCEPT_AND_NOTIFY = 'AcceptAndNotify'
    MSG_FLAG_DISCARD = 'Discard'
    MSG_FLAG_ACCEPT_NOT_NOTIFY = 'AcceptNotNotify'

    WITH_HUGE_GROUPS_YES = 1
    WITH_HUGE_GROUPS_NO = 0

    WITH_NO_ACTIVE_GROUPS_YES = 1
    WITH_NO_ACTIVE_GROUPS_NO = 0

    ONLINE_ONLY_FLAG_YES = 1
    ONLINE_ONLY_FLAG_NO = 0

    def get_app_id_group_list(self):
        """获取 App 中的所有群组

        https://cloud.tencent.com/document/product/269/1614
        """
        return self._get(
            'group_open_http_svc/get_appid_group_list',
            params={
            }
        )

    def create_group(self, name, owner_account=None, member_list=None, group_type=TYPE_PUBLIC, introduction=None,
                     notification=None,
                     face_url=None, max_member_count=None, apply_join_option=JOIN_OPTION_NEED_PERMISSION):
        """创建群组

        https://cloud.tencent.com/document/product/269/1615
        """
        data = {
            'Name': name,
            'Type': group_type,
        }

        if member_list:
            data['MemberList'] = member_list

        if owner_account:
            data['Owner_Account'] = owner_account

        if introduction:
            data['Introduction'] = introduction

        if notification:
            data['Notification'] = notification

        if face_url:
            data['FaceUrl'] = face_url

        if max_member_count:
            data['MaxMemberCount'] = max_member_count

        if apply_join_option:
            data['ApplyJoinOption'] = apply_join_option

        return self._post(
            'group_open_http_svc/create_group',
            data=data
        )

    def get_group_info(self, group_id_list, response_filter=None):
        """获取群组详细资料

        https://cloud.tencent.com/document/product/269/1616
        """
        data = {
            'GroupIdList': group_id_list
        }
        if response_filter:
            data['ResponseFilter'] = response_filter

        return self._post(
            'group_open_http_svc/get_group_info',
            data=data
        )

    def get_group_member_info(self, group_id, limit=None, offset=None, member_info_filter=None):
        """获取群组成员详细资料

        https://cloud.tencent.com/document/product/269/1617
        """
        data = {
            'GroupId': group_id
        }
        if limit:
            data['Limit'] = limit

        if offset:
            data['Offset'] = offset

        if member_info_filter:
            data['MemberInfoFilter'] = member_info_filter

        return self._post(
            'group_open_http_svc/get_group_member_info',
            data=data
        )

    def modify_group_base_info(self, group_id, name=None, introduction=None, notification=None, face_url=None,
                               max_member_num=None, apply_join_option=None, shut_up_all_member=None,
                               app_defined_data=None):
        """修改群组基础资料

        https://cloud.tencent.com/document/product/269/1620
        """
        data = {
            'GroupId': group_id
        }
        if name:
            data['Name'] = name

        if introduction:
            data['Introduction'] = introduction

        if notification:
            data['Notification'] = notification

        if face_url:
            data['FaceUrl'] = face_url

        if max_member_num:
            data['MaxMemberNum'] = max_member_num

        if apply_join_option:
            data['ApplyJoinOption'] = apply_join_option

        if shut_up_all_member:
            data['ShutUpAllMember'] = shut_up_all_member

        if app_defined_data:
            data['AppDefinedData'] = app_defined_data

        return self._post(
            'group_open_http_svc/modify_group_base_info',
            data=data
        )

    def add_group_member(self, group_id, member_list, silence=SILENCE_YES):
        """增加群组成员

        https://cloud.tencent.com/document/product/269/1621
        """
        data = {
            'GroupId': group_id,
            'MemberList': member_list,
            'Silence': silence
        }
        return self._post(
            'group_open_http_svc/add_group_member',
            data=data
        )

    def delete_group_member(self, group_id, member_to_del_account, silence=SILENCE_YES, reason=None):
        """删除群组成员

        https://cloud.tencent.com/document/product/269/1622
        """
        data = {
            'GroupId': group_id,
            'MemberToDel_Account': member_to_del_account,
            'Silence': silence
        }
        if reason:
            data['Reason'] = reason

        return self._post(
            'group_open_http_svc/delete_group_member',
            data=data
        )

    def modify_group_member_info(self, group_id, member_account, role=None, msg_flag=None, name_card=None,
                                 app_member_defined_data=None, shut_up_time=None):
        """修改群成员资料

        https://cloud.tencent.com/document/product/269/1623
        """
        data = {
            'GroupId': group_id,
            'Member_Account': member_account,
        }

        if role:
            data['Role'] = role

        if msg_flag:
            data['MsgFlag'] = msg_flag

        if name_card:
            data['NameCard'] = name_card

        if app_member_defined_data:
            data['AppMemberDefinedData'] = app_member_defined_data

        if shut_up_time:
            data['ShutUpTime'] = shut_up_time

        return self._post(
            'group_open_http_svc/modify_group_member_info',
            data=data
        )

    def destroy_group(self, group_id):
        """解散群组

        https://cloud.tencent.com/document/product/269/1624
        """
        data = {
            'GroupId': group_id,
        }

        return self._post(
            'group_open_http_svc/destroy_group',
            data=data
        )

    def get_joined_group_list(self, member_account, limit=None, offset=None, group_type=None, with_huge_groups=None,
                              with_no_active_groups=None, response_filter=None):
        """获取用户所加入的群组

        https://cloud.tencent.com/document/product/269/1625
        """
        data = {
            'Member_Account': member_account,
        }

        if limit:
            data['Limit'] = limit

        if offset:
            data['Offset'] = offset

        if group_type:
            data['GroupType'] = group_type

        if with_huge_groups:
            data['WithHugeGroups'] = with_huge_groups

        if with_no_active_groups:
            data['WithNoActiveGroups'] = with_no_active_groups

        if response_filter:
            data['ResponseFilter'] = response_filter

        return self._post(
            'group_open_http_svc/get_joined_group_list',
            data=data
        )

    def get_role_in_group(self, group_id, user_account):
        """查询用户在群组中的身份

        https://cloud.tencent.com/document/product/269/1626
        """
        data = {
            'GroupId': group_id,
            'User_Account': user_account
        }

        return self._post(
            'group_open_http_svc/get_role_in_group',
            data=data
        )

    def forbid_send_msg(self, group_id, members_account, shut_up_time=0):
        """批量禁言和取消禁言

        https://cloud.tencent.com/document/product/269/1627
        """
        data = {
            'GroupId': group_id,
            'Members_Account': members_account,
            'ShutUpTime': shut_up_time
        }

        return self._post(
            'group_open_http_svc/forbid_send_msg',
            data=data
        )

    def get_group_shut_uin(self, group_id):
        """获取群组被禁言用户列表

        https://cloud.tencent.com/document/product/269/2925
        """
        data = {
            'GroupId': group_id,
        }

        return self._post(
            'group_open_http_svc/get_group_shutted_uin',
            data=data
        )

    def send_group_msg(self, group_id, msg_body, from_account=None, msg_random=random.randint(0, 4294967295),
                       msg_priority=None, forbid_callback_control=None, online_only_flag=ONLINE_ONLY_FLAG_NO,
                       offline_push_info=None):
        """在群组中发送普通消息

        https://cloud.tencent.com/document/product/269/1629
        """
        data = {
            'GroupId': group_id,
            'Random': msg_random,
            'MsgBody': msg_body
        }

        if from_account:
            data['From_Account']: from_account

        if msg_priority:
            data['MsgPriority']: msg_priority

        if forbid_callback_control:
            data['ForbidCallbackControl']: forbid_callback_control

        if online_only_flag:
            data['OnlineOnlyFlag']: online_only_flag

        if offline_push_info:
            data['OfflinePushInfo']: offline_push_info

        return self._post(
            'group_open_http_svc/send_group_msg',
            data=data
        )

    def send_group_system_notification(self, group_id, content, to_members_account=None):
        """在群组中发送系统通知

        https://cloud.tencent.com/document/product/269/1630
        """
        data = {
            'GroupId': group_id,
            'Content': content,
        }

        if to_members_account:
            data['ToMembers_Account']: to_members_account

        return self._post(
            'group_open_http_svc/send_group_system_notification',
            data=data
        )

    def group_msg_recall(self, group_id, msg_seq_list):
        """撤回群组消息

        https://cloud.tencent.com/document/product/269/12341
        """
        data = {
            'GroupId': group_id,
            'MsgSeqList': msg_seq_list,
        }

        return self._post(
            'group_open_http_svc/group_msg_recall',
            data=data
        )

    def change_group_owner(self, group_id, new_owner_account):
        """转让群组

        https://cloud.tencent.com/document/product/269/1633
        """
        data = {
            'GroupId': group_id,
            'NewOwner_Account': new_owner_account,
        }

        return self._post(
            'group_open_http_svc/change_group_owner',
            data=data
        )

    def import_group(self, name, owner_account=None, group_type=TYPE_PUBLIC, group_id=None, create_time=None,
                     introduction=None, notification=None, face_url=None, max_member_count=None,
                     apply_join_option=None, app_defined_data=None):
        """导入群基础资料

        https://cloud.tencent.com/document/product/269/1634
        """
        data = {
            'Name': name,
            'Type': group_type
        }

        if owner_account:
            data['Owner_Account'] = owner_account

        if create_time:
            data['CreateTime'] = create_time

        if group_id:
            data['GroupId'] = group_id

        if max_member_count:
            data['MaxMemberCount'] = max_member_count

        if introduction:
            data['Introduction'] = introduction

        if notification:
            data['Notification'] = notification

        if face_url:
            data['FaceUrl'] = face_url

        if apply_join_option:
            data['ApplyJoinOption'] = apply_join_option

        if app_defined_data:
            data['AppDefinedData'] = app_defined_data

        return self._post(
            'group_open_http_svc/import_group',
            data=data
        )

    def import_group_msg(self, group_id, msg_list):
        """导入群消息

        https://cloud.tencent.com/document/product/269/1635
        """
        data = {
            'GroupId': group_id,
            'MsgList': msg_list
        }

        return self._post(
            'group_open_http_svc/import_group_msg',
            data=data
        )

    def import_group_member(self, group_id, member_list, join_time=None, unread_msg_num=None):
        """导入群成员

        https://cloud.tencent.com/document/product/269/1636
        """
        data = {
            'GroupId': group_id,
            'MemberList': member_list
        }

        if join_time:
            data['JoinTime'] = join_time

        if unread_msg_num:
            data['UnreadMsgNum'] = unread_msg_num

        return self._post(
            'group_open_http_svc/import_group_member',
            data=data
        )

    def set_unread_msg_num(self, group_id, member_account, unread_msg_num):
        """设置成员未读消息计数

        https://cloud.tencent.com/document/product/269/1637
        """
        data = {
            'GroupId': group_id,
            'Member_Account': member_account,
            'UnreadMsgNum': unread_msg_num,
        }

        return self._post(
            'group_open_http_svc/set_unread_msg_num',
            data=data
        )

    def delete_group_msg_by_sender(self, group_id, sender_account):
        """删除指定用户发送的消息

        https://cloud.tencent.com/document/product/269/2359
        """
        data = {
            'GroupId': group_id,
            'Sender_Account': sender_account,
        }

        return self._post(
            'group_open_http_svc/delete_group_msg_by_sender',
            data=data
        )

    def group_msg_get_simple(self, group_id, req_msg_number, req_msg_seq=None):
        """拉取群漫游消息

        https://cloud.tencent.com/document/product/269/2738
        """
        data = {
            'GroupId': group_id,
            'ReqMsgNumber': req_msg_number,
        }
        if req_msg_seq:
            data['ReqMsgSeq'] = req_msg_seq

        return self._post(
            'group_open_http_svc/group_msg_get_simple',
            data=data
        )



from setuptools import setup, find_packages

setup(
    name="timsdk",
    version='1.1.12',
    packages=find_packages(exclude=('tests', 'tests.*')),
    description="A package for qcloud im(tim), Implement REST API.",
    author="Severus Well",
    author_email='thinkweiwei@msn.com',
    url="https://github.com/WUWUTech/timsdk",
    keywords=['qcloud', 'Tencent', 'im', 'tim', 'SDK'],
    classifiers=[
        'Environment :: Console',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    install_requires=[
        "enum34>=1.1.4; python_version < '3.4'",
        'tls-sig-api-v2>=1.1',
        'requests>=2.4.3'
    ]
)



# -*- coding: utf-8 -*-
# Author: Severus Well
# Contact: thinkweiwei@msn.com
#
# References
#   https://cloud.tencent.com/product/im/developer

from .tls_sig import TLSSig
from .http_client import HttpClient
from .tim_client import TimClient



# -*- coding: utf-8 -*-

# Author: Severus Well
# Contact: thinkweiwei@msn.com

"""A package for qcloud im(tim), Implement REST API."""

from . import HttpClient, TLSSig, api


class TimClient(object):

    def __init__(self, app_id, app_key, identifier='administrator'):
        self._app_id = app_id
        self._app_key = app_key
        self._identifier = identifier
        self._signature = TLSSig.gen_sig(identifier=identifier, app_id=app_id, app_key=app_key)

        client = HttpClient(self._app_id, self._identifier, self._signature)  # Default
        self.group_open_http_svc = api.GroupOpenHttpSVC(client=client)
        self.im_open_login_svc = api.IMOpenLoginSVC(client=client)
        self.open_im = api.OpenIM(client=client)
        self.profile = api.Profile(client=client)
        self.sns = api.SNS(client=client)
        self.open_config_svr = api.OpenConfigSVR(client=client)
        self.open_msg_svc = api.OpenMSGSVC(client=client)

    @property
    def identifier(self):
        return self._identifier

    @identifier.setter
    def identifier(self, value):
        if not isinstance(value, str):
            raise ValueError('identifier must be an str!')
        self._identifier = value

    @property
    def signature(self):
        return self._signature

    @signature.setter
    def signature(self, value):
        if not isinstance(value, str):
            raise ValueError('signature must be an str!')
        self._signature = value
